@@ -208,10 +208,10 @@ int handle_error(t_session *session, int error_code) {
 		memcpy(session->file_on_disk, session->host->website_root, length + 1);
 		strcpy(session->file_on_disk + length, error_handler);
 
-		session->cgi_type = none;
+		session->cgi_type = no_cgi;
 		check_target_is_cgi(session);
 
-		if (session->cgi_type != none) {
+		if (session->cgi_type != no_cgi) {
 			result = execute_cgi(session);
 		} else switch (is_directory(session->file_on_disk)) {
 			case error:
@@ -234,13 +234,13 @@ int handle_error(t_session *session, int error_code) {
 
 	switch (result) {
 		case 301:
-			log_file_error(session, session->file_on_disk, "ErrorHandler is a directory");
+			log_error(session, "ErrorHandler is a directory");
 			break;
 		case 403:
-			log_file_error(session, session->file_on_disk, "no access to ErrorHandler");
+			log_error(session, "no access to ErrorHandler");
 			break;
 		case 404:
-			log_file_error(session, session->file_on_disk, "ErrorHandler not found");
+			log_error(session, "ErrorHandler not found");
 			break;
 		case 500:
 			log_file_error(session, error_handler, "internal error for ErrorHandler");
@@ -254,70 +254,46 @@ int handle_error(t_session *session, int error_code) {
 	return result;
 }
 
-/* Handle the request from a client.
+/* Run RunOnAlter script after PUT or DELETE
  */
-int handle_request(t_session *session) {
-	int retval = 200;
+int run_alter_program(t_session *session) {
+	pid_t pid;
+	char ip[MAX_IP_STR_LEN];
 
-	if ((strcmp(session->method, "GET") == 0) || session->head_request) {
-		if (session->cgi_type != none) {
-			session->body = NULL;
-			retval = execute_cgi(session);
+	if ((pid = fork()) == 0) {
+		if (setsid() != -1) {
+			setenv("REQUEST_METHOD", session->method, 1);
+			setenv("DOCUMENT_ROOT", session->host->website_root, 1);
+			setenv("REQUEST_URI", session->request_uri, 1);
+			if (session->remote_user != NULL) {
+				setenv("REMOTE_USER", session->remote_user, 1);
+			}
+			if (inet_ntop(session->ip_address.family, &(session->ip_address.value), ip, MAX_IP_STR_LEN) != NULL) {
+				setenv("REMOTE_ADDR", ip, 1);
+			}
+			execlp(session->host->run_on_alter, session->host->run_on_alter, (char*)NULL);
+			log_file_error(session, session->host->run_on_alter, "couldn't execute RunOnAlter");
 		} else {
-			retval = send_file(session);
+			log_file_error(session, session->host->run_on_alter, "RunOnAlter setsid() error");
 		}
-		if (retval == 404) {
-			if (session->host->show_index && (session->uri[session->uri_len - 1] == '/')) {
-				retval = show_index(session);
+		exit(EXIT_FAILURE);
+	} else if (pid == -1) {
+		log_file_error(session, session->host->run_on_alter, "RunOnAlter fork() error");
+		return -1;
+#ifdef OPENBSD
+	} else {
+		int loop = session->host->time_for_cgi * 4;
+
+		while (loop > 0) {
+			usleep(250000);
+			if (waitpid(pid, NULL, WNOHANG) > 0) {
+				break;
 			}
 		}
-	} else if (strcmp(session->method, "POST") == 0) {
-		if (session->cgi_type != none) {
-			retval = execute_cgi(session);
-		} else {
-			retval = 405;
-		}
-	} else if (strcmp(session->method, "OPTIONS") == 0) {
-		retval = handle_options_request(session);
-	} else if (strcmp(session->method, "TRACE") == 0) {
-		if (session->binding->enable_trace) {
-			retval = handle_trace_request(session);
-		} else {
-			retval = 501;
-		}
-	} else if (strcmp(session->method, "PUT") == 0) {
-		if (session->binding->enable_alter) {
-			retval = handle_put_request(session);
-		} else {
-			retval = 501;
-		}
-	} else if (strcmp(session->method, "DELETE") == 0) {
-		if (session->binding->enable_alter) {
-			retval = handle_delete_request(session);
-		} else {
-			retval = 501;
-		}
-	} else if (strcmp(session->method, "CONNECT") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "PROPFIND") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "PROPPATCH") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "MKCOL") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "COPY") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "MOVE") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "LOCK") == 0) {
-		retval = 501;
-	} else if (strcmp(session->method, "UNLOCK") == 0) {
-		retval = 501;
-	} else {
-		retval = 400;
+#endif
 	}
 
-	return retval;
+	return 0;
 }
 
 /* Read the request from a client socket.
@@ -352,8 +328,8 @@ int fetch_request(t_session *session) {
 					header_length = strstart + 4 - session->request;
 					session->header_length = header_length;
 
-					session->put_method = (strncmp(session->request, "PUT ", 4) == 0);
-					store_on_disk = (session->put_method && session->binding->enable_alter);
+					determine_request_method(session);
+					store_on_disk = (session->request_method == PUT) && session->binding->enable_alter;
 
 					if (store_on_disk) {
 	 					if ((session->uploaded_file = (char*)malloc(session->config->upload_directory_len + 15)) != NULL) {
@@ -591,11 +567,9 @@ int parse_request(t_session *session, int total_bytes) {
 		return 400;
 	}
 	*(str_end++) = '\0';
-#ifdef HAVE_REWRITE
 	if ((session->request_uri = strdup(session->uri)) == NULL) {
 		return -1;
 	}
-#endif
 
 	/* Protocol version
 	 */
@@ -655,10 +629,6 @@ int parse_request(t_session *session, int total_bytes) {
 		session->kept_alive++;
 	}
 
-	if (strcmp(session->method, "HEAD") == 0) {
-		session->head_request = true;
-	}
-
 	return retval;
 }
 
@@ -764,7 +734,7 @@ t_access allow_client(t_session *session) {
 		return deny;
 	}
 
-	return unknown;
+	return unspecified;
 }
 
 int get_path_info(t_session *session) {
@@ -801,7 +771,7 @@ int get_path_info(t_session *session) {
 	session->cgi_handler = old_cgi_handler;
 	session->cgi_type = old_cgi_type;
 
-	if (cgi_type != none) {
+	if (cgi_type != no_cgi) {
 		if ((session->path_info = strdup(slash)) == NULL) {
 			return -1;
 		}
@@ -840,6 +810,21 @@ int serve_client(t_session *session) {
 		return result;
 	}
 
+	if (session->request_method == TRACE) {
+		if (session->binding->enable_trace == false) {
+			return 501;
+		}
+		return handle_trace_request(session);
+	} else if ((session->request_method == PUT) || (session->request_method == DELETE)) {
+		if (session->binding->enable_alter == false) {
+			return 501;
+		}
+	} else if (session->request_method == unsupported) {
+		return 501;
+	} else if (session->request_method == unknown) {
+		return 400;
+	}
+
 	if (session->hostname != NULL) {
 		if ((host_record = get_hostrecord(session->config->first_host, session->hostname, session->binding)) != NULL) {
 			session->host = host_record;
@@ -850,23 +835,26 @@ int serve_client(t_session *session) {
 #ifdef HAVE_REWRITE
 	/* URL rewrite
 	 */
-	for (i = 0; i < session->host->rewrite_rules.size; i++) {
-		if ((result = rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri)) == RW_ERROR) {
-			return 500;
-		}
-		if (new_uri != NULL) {
-			if (register_tempdata(&(session->tempdata), new_uri, tc_data) == -1) {
-				free(new_uri);
+	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
+		for (i = 0; i < session->host->rewrite_rules.size; i++) {
+			if ((result = rewrite_url(session->uri, session->host->rewrite_rules.item[i], session->config->url_rewrite, &new_uri)) == RW_ERROR) {
 				return 500;
 			}
+			if (new_uri != NULL) {
+				if (register_tempdata(&(session->tempdata), new_uri, tc_data) == -1) {
+					free(new_uri);
+					return 500;
+				}
 #ifdef DEBUG
-			fprintf(stderr, "Rewrite URL: [%s] --> [%s]\n", session->uri, new_uri);
+				fprintf(stderr, "Rewrite URL: [%s] --> [%s]\n", session->uri, new_uri);
 #endif
-			session->uri = new_uri;
-		}
+				session->uri = new_uri;
+			}
 
-		if (result == RW_DENY_ACCESS) {
-			return 403;
+			if (result == RW_DENY_ACCESS) {
+				log_error(session, "access denied via URL rewrite rule");
+				return 403;
+			}
 		}
 	}
 #endif
@@ -883,18 +871,15 @@ int serve_client(t_session *session) {
 		qmark++;
 	}
 
-	session->uri_len = strlen(session->uri);
 	url_decode(session->uri);
+	session->uri_len = strlen(session->uri);
 	if ((session->vars != NULL) && (session->host->secure_url)) {
 		remove_dangerous_chars(session->vars);
 	}
 
 	if (valid_uri(session->uri) == false) {
-		if (session->put_method) {
-			if (session->binding->enable_alter) {
-				return 403;
-			}
-			return 501;
+		if (session->request_method == PUT) {
+			return 403;
 		}
 		return 404;
 	} else if ((result = uri_to_path(session)) != 200) {
@@ -953,8 +938,8 @@ int serve_client(t_session *session) {
 		case allow:
 			break;
 		case pwd:
-		case unknown:
-			if (http_authentication_oke(session, access == unknown) == false) {
+		case unspecified:
+			if (http_authentication_oke(session, access == unspecified) == false) {
 				return 401;
 			}
 	}
@@ -966,7 +951,7 @@ int serve_client(t_session *session) {
 			session->uri_is_dir = true;
 			break;
 		case no:
-			if ((session->put_method == false) && (session->host->enable_path_info)) {
+			if ((session->request_method != PUT) && (session->host->enable_path_info)) {
 				if (get_path_info(session) == -1) {
 					return 500;
 				}
@@ -984,19 +969,23 @@ int serve_client(t_session *session) {
 			log_error(session, fb_filesystem);
 			return 403;
 		case not_found:
-			if (session->host->use_gz_file) {
-				if ((search = get_headerfield("Accept-Encoding:", session->headerfields)) != NULL) {
-					if ((strstr(search, "gzip")) != NULL) {
-						session->accept_gzip = true;
+			if (session->request_method == DELETE) {
+				return 404;
+			} else if (session->request_method != PUT) {
+				if (session->host->use_gz_file) {
+					if ((search = get_headerfield("Accept-Encoding:", session->headerfields)) != NULL) {
+						if ((strstr(search, "gzip")) != NULL) {
+							session->accept_gzip = true;
+						}
 					}
 				}
-			}
 
-			if ((session->accept_gzip == false) && (session->put_method == false)) {
-				if (session->host->fast_cgi.size == 0) {
-					return 404;
-				} else {
-					file_not_found = true;
+				if (session->accept_gzip == false) {
+					if (session->host->fast_cgi.size == 0) {
+						return 404;
+					} else {
+						file_not_found = true;
+					}
 				}
 			}
 	}
@@ -1012,15 +1001,57 @@ int serve_client(t_session *session) {
 		return 301;
 	}
 
-	check_target_is_cgi(session);
-
-	if (file_not_found && (session->cgi_type != fastcgi)) {
-		return 404;
+	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
+		check_target_is_cgi(session);
+		if (file_not_found && (session->cgi_type != fastcgi)) {
+			return 404;
+		}
 	}
 
-	return handle_request(session);
+	switch (session->request_method) {
+		case GET:
+		case HEAD:
+			if (session->cgi_type != no_cgi) {
+				session->body = NULL;
+				result = execute_cgi(session);
+			} else {
+				result = send_file(session);
+			}
+			if (result == 404) {
+				if (session->host->show_index && (session->uri[session->uri_len - 1] == '/')) {
+					result = show_index(session);
+				}
+			}
+			break;
+		case POST:
+			if (session->cgi_type != no_cgi) {
+				result = execute_cgi(session);
+			} else {
+				result = 405;
+			}
+			break;
+		case OPTIONS:
+			result = handle_options_request(session);
+			break;
+		case PUT:
+			result = handle_put_request(session);
+			if (((result == 201) || (result == 204)) && (session->host->run_on_alter != NULL)) {
+				run_alter_program(session);
+			}
+			break;
+		case DELETE:
+			result = handle_delete_request(session);
+			if ((result == 204) && (session->host->run_on_alter != NULL)) {
+				run_alter_program(session);
+			}
+			break;
+		default:
+			result = 400;
+	}
+
+	return result;
 }
-	
+
 /* Request has been handled, handle the return code.
  */
 void handle_request_result(t_session *session, int result) {
@@ -1443,7 +1474,6 @@ int accept_connection(t_binding *binding, t_config *config) {
 /* Run the Hiawatha webserver.
  */
 int run_server(t_settings *settings) {
-	char               *main_configfile = "httpd.conf";
 	int                highest_fd;
 	bool               must_quit = false;
 #ifdef HAVE_COMMAND
@@ -1481,7 +1511,7 @@ int run_server(t_settings *settings) {
 	} else if (settings->config_check) {
 		printf("Using %s\n", settings->config_dir);
 	}
-	if (read_main_configfile(main_configfile, config, settings->config_check) == -1) {
+	if (read_main_configfile("httpd.conf", config, settings->config_check) == -1) {
 		return EXIT_FAILURE;
 	} else if (check_configuration(config) == -1) {
 		return EXIT_FAILURE;
