@@ -5,7 +5,7 @@
 #include "global.h"
 #include "libip.h"
 
-typedef enum { deny, allow, pwd, unknown } t_access;
+typedef enum { deny, allow, pwd, unspecified } t_access;
 typedef enum { tc_data, tc_charlist, tc_accesslist, tc_keyvalue } t_tempdata_type;
 
 typedef struct type_charlist {
