@@ -3,6 +3,7 @@
 #include <stdlib.h>
 #include <stdbool.h>
 #include <string.h>
+#include "alternative.h"
 
 /* Convert a string to a integer.
  */
@@ -138,6 +139,12 @@ bool valid_uri(char *uri) {
 		return false;
 	}
 
+#ifdef CYGWIN
+	if (*(uri + strlen(uri) - 1) == '.') {
+		return false;
+	}
+#endif
+
 	do {
 		if (*(uri + 1) == '.') {
 			if ((*uri == '.') || (*uri == '/')) {
@@ -221,7 +228,11 @@ void remove_dangerous_chars(char *line) {
 int prevent_CMDi(char *line) {
 	int result = 0;
 	short low, high;
-	char value;
+	char value, *prot;
+
+	if ((prot = strcasestr(line, "http://")) != NULL) {
+		memcpy(prot, "XXXX", 4);
+	}
 
 	if (line != NULL) {
 		while (*line != '\0') {
