@@ -110,20 +110,30 @@ void log_system(t_session *session, char *mesg) {
 	}
 }
 
-/* Log an error
+/* Log an error for a specific file
  */
-void log_error(t_session *session, char *mesg) {
+void log_file_error(t_session *session, char *file, char *mesg) {
 	FILE *fp;
 	char str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 
 	if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
 		print_timestamp(str);
 		print_ip_address(str + strlen(str), &(session->ip_address));
-		fprintf(fp, "%s%s\n", str, mesg);
+		if (file != NULL) {
+			fprintf(fp, "%s%s|%s\n", str, file, mesg);
+		} else {
+			fprintf(fp, "%s%s\n", str, mesg);
+		}
 		fclose(fp);
 	}
 }
 
+/* Log an error
+ */
+void log_error(t_session *session, char *mesg) {
+	log_file_error(session, session->file_on_disk, mesg);
+}
+
 /* Log a HTTP request.
  */
 void log_request(t_session *session, int code) {
@@ -289,7 +299,7 @@ void log_unban(char *logfile, t_ip_addr *ip_address, unsigned long connect_attem
 
 /* Log a CGI error.
  */
-void log_file_error(t_session *session, char *file, char *mesg) {
+void log_cgi_error(t_session *session, char *mesg) {
 	FILE *fp;
 	char *c, str[TIMESTAMP_SIZE + IP_ADDRESS_SIZE];
 	int len = 0;
@@ -312,7 +322,7 @@ void log_file_error(t_session *session, char *file, char *mesg) {
 		if ((fp = fopen(session->host->error_logfile, "a")) != NULL) {
 			print_timestamp(str);
 			print_ip_address(str + strlen(str), &(session->ip_address));
-			fprintf(fp, "%s%s|%s\n", str, file, secure_string(mesg));
+			fprintf(fp, "%s%s|%s\n", str, session->file_on_disk, secure_string(mesg));
 			fclose(fp);
 		}
 	}
@@ -331,7 +341,6 @@ void log_close(t_host *host, bool force_close) {
 		while (host != NULL) {
 			if ((now >= host->access_time + LOGFILE_OPEN_TIME) || force_close) {
 				if (*(host->access_fp) != NULL) {
-					//fflush(*(host->access_fp));
 					fclose(*(host->access_fp));
 					*(host->access_fp) = NULL;
 				}
