@@ -56,11 +56,13 @@ static t_host *new_host(void) {
 		init_charlist(&(host->hostname));
 		host->user_websites      = false;
 		host->execute_cgi        = false;
+		host->time_for_cgi       = 5;
 		host->show_index         = false;
 		host->use_gz_file        = false;
 		host->access_list        = NULL;
 		host->alter_list         = NULL;
 		host->alter_fmode        = 0640;
+		host->run_on_alter       = NULL;
 		host->login_message      = "Private page";
 		host->passwordfile       = NULL;
 		host->groupfile          = NULL;
@@ -231,7 +233,6 @@ t_config *default_config(void) {
 		init_charlist(&(config->cgi_extension));
 		config->total_connections  = 100;
 		config->connections_per_ip = 10;
-		config->time_for_cgi       = 5;
 		config->kill_timedout_cgi  = true;
 		config->first_host         = new_host();
 		config->mimetype           = NULL;
@@ -882,10 +883,6 @@ static bool system_setting(char *key, char *value, t_config *config) {
 				}
 			}
 		}
-	} else if (strcmp(key, "timeforcgi") == 0) {
-		if ((config->time_for_cgi = str2int(value)) >= 1) {
-			return true;
-		}
 	} else if (strcmp(key, "uploaddirectory") == 0) {
 		if (valid_directory(value)) {
 			if ((config->upload_directory = strdup(value)) != NULL) {
@@ -1011,6 +1008,10 @@ static bool user_setting(char *key, char *value, t_host *host, t_tempdata **temp
 			return true;
 		}
 #endif
+	} else if (strcmp(key, "runonalter") == 0) {
+		if ((host->run_on_alter = strdup(value)) != NULL) {
+			return true;
+		}
 	} else if (strcmp(key, "setenv") == 0) {
 		if (parse_keyvaluelist(value, &(host->envir_str), "=") != -1) {
 			if (register_tempdata(tempdata, host->envir_str, tc_keyvalue) != -1) {
@@ -1103,7 +1104,7 @@ static bool host_setting(char *key, char *value, t_host *host) {
 			return true;
 		}
 	} else if (strcmp(key, "hostname") == 0) {
-		if (parse_charlist(strlower(value), &(host->hostname)) == 0) {
+		if (parse_charlist(value, &(host->hostname)) == 0) {
 			return true;
 		}
 	} else if (strcmp(key, "imagereferer") == 0) {
@@ -1140,6 +1141,10 @@ static bool host_setting(char *key, char *value, t_host *host) {
 		if (parse_yesno(value, &(host->secure_url)) == 0) {
 			return true;
 		}
+	} else if (strcmp(key, "timeforcgi") == 0) {
+		if ((host->time_for_cgi = str2int(value)) >= 1) {
+			return true;
+		}
 	} else if (strcmp(key, "triggeroncgistatus") == 0) {
 		if (parse_yesno(value, &(host->trigger_on_cgi_status)) == 0) {
 			return true;
@@ -1832,10 +1837,10 @@ t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding) {
 						continue;
 					}
 				}
-				if (strncmp(hostname, *(host->hostname.item + i), len_hostname) == 0) {
+				if (strncasecmp(hostname, *(host->hostname.item + i), len_hostname) == 0) {
 					/* Exact match */
 					return host;
-				} else if (strncmp(*(host->hostname.item + i), "*.", 2) == 0) {
+				} else if (strncasecmp(*(host->hostname.item + i), "*.", 2) == 0) {
 					/* Wildcard in configuration */
 					if (strncmp(hostname, *(host->hostname.item + i) + 2, len_hostname) == 0) {
 						/* Only domainname requested */
@@ -1843,7 +1848,7 @@ t_host *get_hostrecord(t_host *host, char *hostname, t_binding *binding) {
 					} else {
 						len = strlen(*(host->hostname.item + i));
 						if (len_hostname >= len) {
-							if (strncmp(hostname + len_hostname - len + 1, *(host->hostname.item + i) + 1, len - 1) == 0) {
+							if (strncasecmp(hostname + len_hostname - len + 1, *(host->hostname.item + i) + 1, len - 1) == 0) {
 								/* Wildcard match for hostname */
 								return host;
 							}
