@@ -337,7 +337,7 @@ int send_file(t_session *session) {
 		}
 
 		retval = 200;
-		if (session->head_request == false) {
+		if (session->request_method != HEAD) {
 			if (is_volatile_object(session) && (file_size <= MAX_VOLATILE_SIZE)) {
 				/* volatile object
 				 */
@@ -653,7 +653,7 @@ int execute_cgi(t_session *session) {
 
 	cgi_info.select_timeout.tv_sec = 1;
 	cgi_info.select_timeout.tv_usec = 0;
-	cgi_info.timer = session->config->time_for_cgi;
+	cgi_info.timer = session->host->time_for_cgi;
 
 	do {
 #ifdef DEBUG
@@ -671,11 +671,11 @@ int execute_cgi(t_session *session) {
 
 		switch (cgi_result) {
 			case cgi_ERROR:
-				log_file_error(session, session->file_on_disk, "error while executing CGI");
+				log_error(session, "error while executing CGI");
 				retval = 500;
 				break;
 			case cgi_TIMEOUT:
-				log_file_error(session, session->file_on_disk, "CGI timeout");
+				log_error(session, "CGI timeout");
 				if (in_body) {
 					retval = rs_DISCONNECT;
 				} else {
@@ -696,7 +696,7 @@ int execute_cgi(t_session *session) {
 					/* Error received from CGI
 					 */
 					*(cgi_info.error_buffer + cgi_info.error_len) = '\0';
-					log_file_error(session, session->file_on_disk, cgi_info.error_buffer);
+					log_cgi_error(session, cgi_info.error_buffer);
 					cgi_info.error_len = 0;
 				}
 
@@ -706,7 +706,7 @@ int execute_cgi(t_session *session) {
 					if (in_body) {
 						/* Read content
 						 */
-						if (session->head_request == false) {
+						if (session->request_method != HEAD) {
 							if (send_in_chunks) {
 								if (send_chunk(session, cgi_info.input_buffer, cgi_info.input_len) == -1) {
 									retval = rs_DISCONNECT;
@@ -757,7 +757,7 @@ int execute_cgi(t_session *session) {
 								}
 
 								if (retval <= 0) {
-									log_file_error(session, session->file_on_disk, "invalid status code received from CGI");
+									log_error(session, "invalid status code received from CGI");
 									retval = 500;
 									break;
 								}
@@ -791,7 +791,7 @@ int execute_cgi(t_session *session) {
 
 							/* Send the header.
 							 */
-							if (send_in_chunks || session->head_request) {
+							if (send_in_chunks || (session->request_method == HEAD)) {
 								new_line += 4;
 								result = new_line - cgi_info.input_buffer;
 								if (send_buffer(session, cgi_info.input_buffer, result) == -1) {
@@ -802,7 +802,7 @@ int execute_cgi(t_session *session) {
 									retval = rs_DISCONNECT;
 									break;
 								}
-								if (session->head_request == false) {
+								if (session->request_method != HEAD) {
 									result = cgi_info.input_len - result;
 									if (result > 0) {
 										if (send_chunk(session, new_line, result) == -1) {
@@ -826,7 +826,7 @@ int execute_cgi(t_session *session) {
 							in_body = true;
 							cgi_info.input_len = 0;
 						} else if (cgi_info.input_len > MAX_CGI_HEADER) {
-							log_file_error(session, session->file_on_disk, "CGI's HTTP header too large");
+							log_error(session, "CGI's HTTP header too large");
 							retval = 500;
 							break;
 						}
@@ -843,11 +843,11 @@ int execute_cgi(t_session *session) {
 					}
 				} else {
 					if (cgi_info.input_len == 0) {
-						log_file_error(session, session->file_on_disk, "no output");
+						log_error(session, "no output");
 					} else if (strstr(cgi_info.input_buffer, "\r\n\r\n") == NULL) {
-						log_file_error(session, session->file_on_disk, "non-RFC HTTP header ('\\n' instead of '\\r\\n')");
+						log_error(session, "non-RFC HTTP header ('\\n' instead of '\\r\\n')");
 					} else {
-						log_file_error(session, session->file_on_disk, "CGI only printed a HTTP header, no content");
+						log_error(session, "CGI only printed a HTTP header, no content");
 					}
 					retval = 500;
 				}
@@ -962,7 +962,7 @@ int show_index(t_session *session) {
 		return -1;
 	}
 
-	if (session->head_request) {
+	if (session->request_method == HEAD) {
 		send_buffer(session, "\r\n", 2);
 		return 200;
 	}
@@ -1189,7 +1189,7 @@ int handle_options_request(t_session *session) {
 		return -1;
 	} else if (send_buffer(session, hs_allow, 25) == -1) {
 		return -1;
-	} else if (session->cgi_type != none) {
+	} else if (session->cgi_type != no_cgi) {
 		if (send_buffer(session, hs_post, 6) == -1) {
 			return -1;
 		}
@@ -1300,7 +1300,7 @@ static t_access allow_alter(t_session *session) {
 		return deny;
 	}
 
-	return unknown;
+	return unspecified;
 }
 
 int handle_put_request(t_session *session) {
@@ -1319,7 +1319,7 @@ int handle_put_request(t_session *session) {
 	 */
 	switch (allow_alter(session)) {
 		case deny:
-		case unknown:
+		case unspecified:
 			log_error(session, fb_alterlist);
 			return 403;
 		case allow:
@@ -1349,7 +1349,7 @@ int handle_put_request(t_session *session) {
 			return 416;
 		}
 		if ((handle_write = open(session->file_on_disk, O_CREAT | O_WRONLY)) == -1) {
-			log_file_error(session, session->file_on_disk, fb_filesystem);
+			log_error(session, fb_filesystem);
 			return 403;
 		}
 		file_size = NEW_FILE;
@@ -1379,7 +1379,7 @@ int handle_put_request(t_session *session) {
 			lock_timeout--;
 			sleep(1);
 		} else {
-			log_file_error(session, session->file_on_disk, "can't lock file for writing (PUT)");
+			log_error(session, "can't lock file for writing (PUT)");
 			close(handle_write);
 			if (file_size == NEW_FILE) {
 				unlink(session->file_on_disk);
@@ -1391,7 +1391,7 @@ int handle_put_request(t_session *session) {
 	/* Change file mode
 	 */
 	if (fchmod(handle_write, session->host->alter_fmode) == -1) {
-		log_file_error(session, session->file_on_disk, "can't change file mode (PUT)");
+		log_error(session, "can't change file mode (PUT)");
 	}
 
 	/* Handle upload range
@@ -1492,7 +1492,7 @@ int handle_delete_request(t_session *session) {
 	 */
 	switch (allow_alter(session)) {
 		case deny:
-		case unknown:
+		case unspecified:
 			log_error(session, fb_alterlist);
 			return 403;
 		case allow:
